#! /bin/bash

# Comments

# Installer for the Rowland NMR toolkit
# Written by Mark Maciejewski on Sept, 2007
# Installer only works for Linux kernel 2.3 and later and OSX 10.3 and later
# Installation is in three parts
#   1. Install Files
#   2. Edit /etc/csh.cshrc /etc/cshrc /etc/bashrc /etc/profile
#   3. Alter shmmax and shmall parameters if needed
# NOTE: The installer edits the login files for PATH, stacksize, datasize (ulimit/limit), and the number of cpu's
#
# Installer assumes that bash is installed and that a few generic commands exists such as:  
#    id, uname, cp, mv, pwd, rm, date, ulimit, grep, sed, awk, tr, clear, hostinfo(apple)

clear


# Check to see if are root or have Administrative privledges
if [ `id -u` != 0 ]; then
  echo 'You need to be logged in as root to proceed'
  echo 'For linux and OS X with root user enabled use su to change to the root user'
  echo 'For OS X without root user enabled use sudo -s from an account with Administrator privledges'
  exit 1
fi

current_dir=`pwd`

# Add comments at top of output
echo ''
echo 'This installer implements three functions, which may performed individually'
echo '   (1) Installs the RNMRTK program suite'
echo '   (2) Modifies the system shared memory parameters needed to run the RNMRTK programs'
echo '   (3) Modifies the global login files to include the RNMRTK programs in the search path'
echo ''
echo 'This install script assumes the installation will reside on the computer used to run the programs'
echo 'For installation on file servers: First install on local computers, and then copy the rnmrtk folder '
echo 'to the server. Adjustment of the $PATH environmental variable must then be done manually'
echo ''

# Determine if Linux or OSX or manually choose intall type
operating_system=`uname -s`
if [ "$operating_system" = "Linux" ]; then
  echo 'The operating system seems to be Linux'
  echo ''
  install_type="linux"
elif [ "$operating_system" = "Darwin" ]; then
  echo 'The operating system seems to be Darwin (OSX)'
  processor_type=`uname -p`
  echo "The processor type appears to be a $processor_type"
  if [ "$processor_type" = "powerpc" ]; then
    install_type="apple-powerpc"
  else
    install_type="apple-intel"
  fi
else
  echo 'Operating system does not appear to be Linux or OSX'
  install_type="unknown"
fi
if [ "$install_type" != "unknown" ]; then
  read -p "Do you want to proceed with installation for $install_type (y/n)?"
  if [ "$REPLY" = "y" ]; then
    echo ''
  else
    echo 'Aborting install!'
    exit 1
  fi
  #else
  #  read -p "Please choose an install type: (1) linux, (2) apple-powerpc, (3) apple-intel, (4) abort install: "
  #  if [ $REPLY = "1" ]; then
  #    install_type="linux"
  #  elif [ $REPLY = "2" ]; then
  #    install_type="apple-powerpc"
  #  elif [ $REPLY = "3" ]; then
  #    install_type="apple-intel"
  #  else
  #    echo "No installation type was selected. Aborting install!"
  #    exit 1
  #  fi
  #  echo ''
  #fi
else
  read -p "Please choose an install type: (1) linux, (2) apple-powerpc, (3) apple-intel, (4) abort install: "
  if [ "$REPLY" = "1" ]; then
    install_type="linux"
  elif [ "$REPLY" = "2" ]; then
    install_type="apple-powerpc"
  elif [ "$REPLY" = "3" ]; then
    install_type="apple-intel"
  else
    echo 'No installation type was selected. Aborting install!'
    exit 1
  fi
  echo ''
fi




# Linux specific installation process
if [ "$operating_system" = "Linux" ]; then
  
  # Ask if files should be installed
  read -p "STEP 1: Do you want to install RNMRTK files (y/n)?"
  if [ "$REPLY" = "y" ]; then
    
    # Enter installation directory or choose default
    read -p "Enter full path to install directory or hit ENTER for /usr/local: "
    if [ -z "$REPLY" ]; then
      install_path="/usr/local"
    elif [ -d "$REPLY" ]; then
      install_path=$REPLY
    else
      echo "The path $REPLY does not exist. Aborting!"
      exit 1
    fi
    
    # Make sure the rnmrtk.v3.tar file exists
    if [ ! -e rnmrtk.v3.tar ]; then
      echo 'The rnmrtk.v3.tar file is not present. Aborting install!'
      exit 1
    fi
    
    # Check to see if the install path is writable by root
    exists_install_path=`chmod 755 $install_path 2>&1 | tr "\n" " " | sed s/" "//g`
    if [ -z "$exists_install_path" ]; then
      
      # Check to see if an existing rnmrtk folder exists and back it up before proceeding
      if [ -d "$install_path/rnmrtk.v3" ]; then
	read -p "rnmrtk.v3 already exists in this folder. Overwrite [Original will be backed-up] (y/n): "
	if [ "$REPLY" = "y" ]; then
	  DATE=`date +%m%d%Y`
	  cd $install_path
	  if [ -e "rnmrtk.v3.$DATE.tar" ]; then
	    count=`ls rnmrtk.v3.$DATE* | wc -l | sed s/" "/""/g`
	    echo "Backing up existing program as $install_path/rnmrtk.v3.$DATE.$count.tar"
	    echo ''
	    tar -cf rnmrtk.v3.$DATE.$count.tar rnmrtk.v3
	  else
	    echo "Backing up existing program as $install_path/rnmrtk.v3.$DATE.tar"
	    echo ''
	    tar -cf rnmrtk.v3.$DATE.tar rnmrtk.v3
	  fi
	  
	  # Remove the rnmrtk folder after backing it up
	  rm -rf rnmrtk.v3
	  
	  # Copy the rnmrtk program to the install path and unpack, remove the tar file when done.
          cd $current_dir
          cp rnmrtk.v3.tar $install_path
          cd $install_path
          tar -xf rnmrtk.v3.tar
          rm -rf rnmrtk.v3.tar
	else
	  echo 'You have chosen not to install files. Proceeding to Step 2'
	  echo ''
	fi
      fi
    else
      echo "Root does not have write access to $install_path. Aborting!"
      exit 1
    fi
  else
    echo ''
    echo 'You have chosen to not install files at this time'
    echo ''
  fi
  
  
  
  # Step 2 edit login files
  
  # Ask if configuration files should be updated
  read -p "STEP 2: Do you want to make changes to global login files for bash/csh/tcsh shells (y/n)?"
  if [ "$REPLY" = "y" ]; then
    
    # Ask for the path to the rnmrtk.v3 folder if no installation was done in step 1
    if [ -z "$install_path" ]; then
      read -p "What is the full path to the rnmrtk.v3 folder (e.g. /usr/local) ? "
      if [ -z "$REPLY" ]; then
	echo 'You did not provide the path. Aborting!'
	exit 1
      else
	install_path=$REPLY
      fi
      
      # Check to see if rnmrtk.v3 was added to the install path and remove it before proceeding if it is present
      exists_rnmrtk=`echo $install_path | grep "/rnmrtk.v3" | tr "\n" " " | sed s/" "//g`
      if [ ! -z "$exists_rnmrtk" ]; then
        install_path=`echo $install_path | sed 's/\/rnmrtk.v3//'`
      fi
    fi
    
    # Find the current stacksize, datasizem and DATE
    stacksize=`ulimit -s`
    datasize=`ulimit -d`
    DATE=`date +%m%d%Y`
    
    # Find the number of cpu's
    cpu_number=`cat /proc/cpuinfo | grep processor | wc -l`
    
    # Make changes to /etc/profile
    header="no"
    
    # Check to see if /etc/profile exists and is writable by root
    exists_profile=`chmod 755 /etc/profile 2>&1 | tr "\n" " " | sed s/" "//g`
    if [ -z "$exists_profile" ]; then
      
      # Check to see if the install path is already present in the /etc/profile file. do nothing if it is there already
      exists_path=`cat /etc/profile | sed 's/\/\//\//g' | grep $install_path/rnmrtk.v3 | tr "\n" " " | sed s/" "//g`
      if [ -z "$exists_path" ]; then
        
	# Add a header if any changes are made to /etc/profile
	header="yes"
	echo 'Making changes to /etc/profile'
	echo '' >> /etc/profile
        echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/profile
        
	# Add the path to /etc/profile
	echo "PATH=\"$install_path/rnmrtk.v3:\$PATH\"" >> /etc/profile
        echo 'export PATH' >> /etc/profile
      else
        echo ''
      fi
      
      # Make changes for stack size limit if the stacksize is < 32768
      if [ $(echo "$stacksize < 32768" | bc) -eq 1 ]; then
	
	# Test if stacksize has already been set to 32768 in /etc/profile and do nothing if it has
	exists_stacksize=`grep 'ulimit -s 32768' /etc/profile | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_stacksize" ]; then
	  
	  # Add a header if changes are being made and no header was created earlier
	  if [ ! $header = "yes" ]; then
	    echo 'Making changes to /etc/profile'
	    echo '' >> /etc/profile
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/profile
	    header="yes"
	  fi
	  
	  # Make the change to stacksize
	  echo 'ulimit -s 32768' >> /etc/profile
	fi
      fi
      
      # Make changes for data size to make it unlimited
      if [ ! "$datasize" = "unlimited" ]; then
	
	# Test if datasize has already been set to unlimited in /etc/profile and do nothing if it has
	exists_datasize=`grep 'ulimit -d unlimited' /etc/profile | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_datasize" ]; then
	  
	  # Add a header if changes are being made and no header was created earlier
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/profile'
	    echo '' >> /etc/profile
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/profile
	    header="yes"
	  fi
	  
	  # Make changes to datasize
	  echo 'ulimit -d unlimited' >> /etc/profile
	fi
      fi
      
      # Make changes for mp_set_numthreads
      if [ ! "$cpu_number" = 0 ]; then
	
	# Test if mp_set_numthreads has already been set in /etc/profile and do nothing if it has
	exists_mp_set_numthreads=`grep 'MP_SET_NUMTHREADS' /etc/profile | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_mp_set_numthreads" ]; then
	  
	  # Add a header if changes are being made and no header was created earlier
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/profile'
	    echo '' >> /etc/profile
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/profile
	    header="yes"
	  fi
	  
	  # Make changes to mp_set_numthreads
	  echo "MP_SET_NUMTHREADS=$cpu_number" >> /etc/profile
	  echo 'export MP_SET_NUMTHREADS' >> /etc/profile
	fi
      fi
      
    else
      
      echo 'The file /etc/profile does not exist or is not owned by root'
      echo 'Manual changes to /etc/profile must be made for bash/sh users'
      echo ''
    fi
    
    
    # Make changes to /etc/bashrc
    header="no"
    
    # Test if /etc/bashrc exists and is writable by root
    exists_bashrc=`chmod 755 /etc/bashrc 2>&1 | tr "\n" " " | sed s/" "//g`
    if [ -z "$exists_bashrc" ]; then
      
      # Read /etc/bashrc, remove any "//" which may occur in paths, grep for install path, eliminate redundant rnmrtk paths
      #   into a single line, eliminate any spaces so the exists_path variable can be set
      exists_path=`cat /etc/bashrc | sed 's/\/\//\//g' | grep $install_path/rnmrtk.v3 | tr "\n" " " | sed s/" "//g`
      if [ -z "$exists_path" ]; then
        
	# Print header if changes  are going to be made
	header="yes"
	echo 'Making changes to /etc/bashrc'
	echo '' >> /etc/bashrc
        echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/bashrc
        echo "PATH=\"$install_path/rnmrtk.v3:\$PATH\"" >> /etc/bashrc
        
	# Add the path to /etc/bashrc
	echo 'export PATH' >> /etc/bashrc
      else
        echo ""
      fi
     
      # Make changes for stack size limit if < 32768
      if [ $(echo "$stacksize < 32768" | bc) -eq 1 ]; then
	
	# Determine if stacksize is set in /etc/bashrc already and do nothing if it has	
	exists_stacksize=`grep 'ulimit -s 32768' /etc/bashrc | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_stacksize" ]; then
	  
	  # Add header if changes are being made and no header was added earlier
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/bashrc'
	    echo '' >> /etc/bashrc
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/bashrc
	    header="yes"
	  fi
	  
	  # Make the change to stacksize in /etc/bashrc
	  echo 'ulimit -s 32768' >> /etc/bashrc
	fi
      fi
      
      # Make changes for data size limit if not already set to unlimited
      if [ ! "$datasize" = "unlimited" ]; then
	
	# Determine if datasize is set in /etc/bashrc already and do nothing if it has	
	exists_datasize=`grep 'ulimit -d unlimited' /etc/bashrc | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_datasize" ]; then
	 
	  # Add header if changes are being made and no header was added earlier
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/bashrc'
	    echo '' >> /etc/bashrc
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/bashrc
	    header="yes"
	  fi
	 
	  # Make change to datasize in /etc/bashrc
	  echo 'ulimit -d unlimited' >> /etc/bashrc
	fi
      fi
      
      # Make changes for mp_set_numthreads
      if [ ! "$cpu_number" = 0 ]; then
	
	# Test if mp_set_numthreads has already been set in /etc/bashrc and do nothing if it has
	exists_mp_set_numthreads=`grep 'MP_SET_NUMTHREADS' /etc/bashrc | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_mp_set_numthreads" ]; then
	  
	  # Add a header if changes are being made and no header was created earlier
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/bashrc'
	    echo '' >> /etc/bashrc
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/bashrc
	    header="yes"
	  fi
	  
	  # Make changes to mp_set_numthreads
	  echo "MP_SET_NUMTHREADS=$cpu_number" >> /etc/bashrc
	  echo 'export MP_SET_NUMTHREADS' >> /etc/bashrc
	fi
      fi
      
    else
      echo 'The file /etc/bashrc does not exist or is not owned by root'
      echo 'Manual changes to /etc/bashrc must be made for bash/sh users'
      echo ''
    fi
    
    
    # Make changes to /etc/csh.cshrc
    header="no"
    
    # Test if /etc/csh.cshrc exists and is writable by root
    exists_csh_cshrc=`chmod 755 /etc/csh.cshrc 2>&1 | tr "\n" " " | sed s/" "//g`
    if [ -z "$exists_csh_cshrc" ]; then
      
      # Check if the path already is set in /etc/csh.cshrc and do nothing if it has
      exists_path=`cat /etc/csh.cshrc | sed 's/\/\//\//g' | grep $install_path/rnmrtk.v3 | tr "\n" " " | sed s/" "//g`
      if [ -z "$exists_path" ]; then
        
	# Print header if changes are to be made
	header="yes"
	echo 'Making changes to /etc/csh.cshrc'
	echo '' >> /etc/csh.cshrc
        echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/csh.cshrc
        
	# Set the PATH in /etc/csh.cshrc
	echo "setenv PATH \"$install_path/rnmrtk.v3:\$PATH\"" >> /etc/csh.cshrc
      else
        echo ""
      fi
      
      # Make changes for stack size limit if , 32768
      if [ $(echo "$stacksize < 32768" | bc) -eq 1 ]; then
	
	# Determine if stacksize is already being set in csh.cshrc and do nothing if it has
	exists_stacksize=`grep "limit stacksize 32768" /etc/csh.cshrc | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_stacksize" ]; then
	  
	  # Print the header if not already printed above
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/csh.cshrc'
	    echo '' >> /etc/csh.cshrc
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/csh.cshrc
	    header="yes"
	  fi
	  
	  # Make changes for stacksize in csh.cshrc
	  echo 'limit stacksize 32768' >> /etc/csh.cshrc
	fi
      fi
      
      # Make changes for data size limit if needed
      if [ ! "$datasize" = "unlimited" ]; then
	
	# Test if datasize is already set in csh.cshrc and do nothing if it has
	exists_datasize=`grep "limit datasize unlimited" /etc/csh.cshrc | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_datasize" ]; then
	  
	  # Print header if not already printed above
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/csh.cshrc'
	    echo '' >> /etc/csh.cshrc
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/csh.cshrc
	    header="yes"
	  fi
	  
	  # Make change to datasize in csh.cshrc
	  echo 'limit datasize unlimited' >> /etc/csh.cshrc
	fi
      fi
      
      # Make changes for mp_set_numthreads
      if [ ! "$cpu_number" = 0 ]; then
	
	# Test if mp_set_numthreads has already been set in /etc/csh.cshrc and do nothing if it has
	exists_mp_set_numthreads=`grep 'MP_SET_NUMTHREADS' /etc/csh.cshrc | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_mp_set_numthreads" ]; then
	  
	  # Add a header if changes are being made and no header was created earlier
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/csh.cshrc'
	    echo '' >> /etc/csh.cshrc
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/csh.cshrc
	    header="yes"
	  fi
	  
	  # Make changes to mp_set_numthreads
	  echo "setenv MP_SET_NUMTHREADS $cpu_number" >> /etc/csh.cshrc
	fi
      fi
          
    fi
    
    
    # Make changes to /etc/cshrc
    header="no"
    
    # Determine if /etc/chsrc exists and is writable by root
    exists_cshrc=`chmod 755 /etc/cshrc 2>&1 | tr "\n" " " | sed s/" "//g`
    if [ -z "$exists_cshrc" ]; then
      
      # Determine if the path is already set in /etc/cshrc and do nothing if it has
      exists_path=`cat /etc/cshrc | sed 's/\/\//\//g' | grep $install_path/rnmrtk.v3 | tr "\n" " " | sed s/" "//g`
      if [ -z "$exists_path" ]; then
        
	# Print the header if changes are to be made
	header="yes"
	echo 'Making changes to /etc/cshrc'
	echo '' >> /etc/cshrc
        echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/cshrc
        
	# Set the path in cshrc
	echo "setenv PATH \"$install_path/rnmrtk.v3:\$PATH\"" >> /etc/cshrc
      else
        echo ''
      fi
      
      # Make changes for stack size limit if < 32768
      if [ $(echo "$stacksize < 32768" | bc) -eq 1 ]; then
	
	# test if stacksize is set already in cshrc and do nothing if it has
	exists_stacksize=`grep "limit stacksize 32768" /etc/cshrc | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_stacksize" ]; then
	    
	  # Print the header if changes are to be made and it was not done earlier
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/cshrc'
	    echo '' >> /etc/cshrc
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/cshrc
	    header="yes"
	  fi
	  
	  # Make changes to stacksize in cshrc
	  echo 'limit stacksize 32768' >> /etc/cshrc
	fi
      fi
      
      # Make changes for data size limit if not already set to unlimited
      if [ ! "$datasize" = "unlimited" ]; then
	
	# Test if datasize is set in cshrc already and do nothing if it is
	exists_datasize=`grep "limit datasize unlimited" /etc/cshrc | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_datasize" ]; then
	  
	  # Print the header if changes are to be made and it was not done earlier
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/cshrc'
	    echo '' >> /etc/cshrc
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/cshrc
	    header="yes"
	  fi
	  
	  # Make changes to datasize in cshrc
	  echo 'limit datasize unlimited' >> /etc/cshrc
	fi
      fi
      
      # Make changes for mp_set_numthreads
      if [ ! "$cpu_number" = 0 ]; then
	
	# Test if mp_set_numthreads has already been set in /etc/cshrc and do nothing if it has
	exists_mp_set_numthreads=`grep 'MP_SET_NUMTHREADS' /etc/cshrc | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_mp_set_numthreads" ]; then
	  
	  # Add a header if changes are being made and no header was created earlier
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/cshrc'
	    echo '' >> /etc/cshrc
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/cshrc
	    header="yes"
	  fi
	  
	  # Make changes to mp_set_numthreads
	  echo "setenv MP_SET_NUMTHREADS $cpu_number" >> /etc/cshrc
	fi
      fi
      
    fi
    if [ ! -z "$exists_csh_cshrc" ]; then
      if [ ! -z "$exists_cshrc" ]; then
        echo 'Neither /etc/csh.cshrc nor /etc/cshrc exist or are owned by root'
        echo 'Manual changes to /etc/csh.cshrc or /etc/cshrc must be made for csh/tcsh users'
        echo ''
      fi
    fi
    
  else
    echo ''
    echo 'You have chosen to not change login files. You must edit the $PATH variable'
    echo 'manually and make changes to the filesize and stacksize if necessary!'
    echo ''
  fi
        
    
  # Add a slight pause before proceeding to step 3
  sleep 1
  
  # Modify Shared Memory Parameters
  
  # Determine the current shared memory parameters and report their size in a header
  current_shmmax=`cat /proc/sys/kernel/shmmax`
  current_shmall=`cat /proc/sys/kernel/shmall`
  echo ''
  echo '************************** Information on current shared memory parameters **************************'
  echo "The current setting for the boot parameter shmmax is $current_shmmax bytes ($(($current_shmmax/(1024*1024)))) MB"
  echo "With this setting the largest NMR data set that may be processed would be slightly less than $(($current_shmmax/(1024*1024))) MB"
  echo "     That is the size of a typical 2D experiment with $(($current_shmmax/(2048*4*2))) final points in the indirect dimension or"
  x=`echo "sqrt(($current_shmmax/(1024*4*4)))" | bc`
  echo "     a typcial 3D experiment with an average of $x final points in the indirect dimensions."
  echo ''
  
  # Ask if the shared memory paramters should be increased and wait for reply
  read -p "STEP 3: Do you want to change the size of shmmax at this time (y/n)?"
  if [ "$REPLY" = "y" ]; then
    read -p "Choose the shared memory size you would like: (1)256 MB, (2)512 MB, (3)768 MB, (4)1 G, (5)2 G, or (6)4G: "
    new_shmall=2097152
    if [ "$REPLY" = "1" ]; then
      new_shmmax=272629760
    elif [ "$REPLY" = "2" ]; then
      new_shmmax=541065216
    elif [ "$REPLY" = "3" ]; then
      new_shmmax=809500672
    elif [ "$REPLY" = "4" ]; then
      new_shmmax=1077936128
    elif [ "$REPLY" = "5" ]; then
      new_shmmax=2151677952
    elif [ "$REPLY" = "6" ]; then
      new_shmmax=4293918720
    else
      echo 'An invalid entry was entered. Aborting install!'
      exit 1
    fi
   
    # Checking to see if new shmmax value choosen is larger than the existing shmmax value.
    if [ $(echo "$current_shmmax == $new_shmmax" | bc) -eq 1 ]; then
      echo "The current shmmax value is the same as the requested value. No change is necessary"
      echo ''
      echo 'Installation complete!'
      echo ''
      exit 1
    fi
    
    # Check to see if the new shmmax value is larger than the existing value. If not ask if we should continue
    if [ $(echo "$current_shmmax > $new_shmmax" | bc) -eq 1 ]; then
      read -p "The current shmmax value is larger than the requested size. Do you want to continue and make shmmax smaller (y/n)?"
      if [ "$REPLY" = "y" ]; then
        echo "Adjusting shmmax to a smaller value"
      else
	echo ''
	echo 'Installation complete!'
	echo ''
	exit 1
      fi
    fi  
    
    # Determine if shared memory is handled by sysctl. rc.local is another possibility explored later
    if [ -e "/sbin/sysctl" ]; then
      if [ -e "/etc/sysctl.conf" ]; then
        # Make a backup copy of /etc/sysctl.conf or create the file if it does not exist
	DATE=`date +%m%d%Y`
	if [ -e "/etc/sysctl.conf.$DATE" ]; then
	  count=`ls /etc/sysctl.conf.$DATE* | wc -l | sed s/" "/""/g`
	  echo "A backup copy of /etc/sysctl.conf is being saved as /etc/sysctl.conf.$DATE.$count"
	  cp /etc/sysctl.conf /etc/sysctl.conf.$DATE.$count
	else
	  echo "A backup copy of /etc/sysctl.conf is being saved as /etc/sysctl.conf.$DATE"
	  cp /etc/sysctl.conf /etc/sysctl.conf.$DATE
	fi
      else
        touch /etc/sysctl.conf
      fi
      
      # test if sysctl.conf exists and is writable by root
      exists_sysctl_conf=`chmod 644 /etc/sysctl.conf 2>&1 | tr "\n" " " | sed s/" "//g`
      if [ -z "$exists_sysctl_conf" ]; then
	
	# Check if the values for shmmax and shmall already appear in /etc/sysctl.conf
	exists_shmmax=`grep kernel.shmmax= /etc/sysctl.conf | tr "\n" " " | sed s/" "//g`
	exists_shmall=`grep kernel.shmall= /etc/sysctl.conf | tr "\n" " " | sed s/" "//g`
	
	# If shmmax does not exist in /etc/sysctl.conf add it here
	if [ -z "$exists_shmmax" ]; then
	  echo '' >> /etc/sysctl.conf
	  echo '# Permit large shared memory sections' >> /etc/sysctl.conf
	  echo "kernel.shmmax=$new_shmmax" >> /etc/sysctl.conf
	else
          
	  # If shmmax already appears in /etc/sysctl.conf update the value here
	  sysctl_conf_shmmax=`grep kernel.shmmax= /etc/sysctl.conf | sed s/kernel.shmmax=//`
          cat /etc/sysctl.conf | sed s/kernel.shmmax=$sysctl_conf_shmmax/kernel.shmmax=$new_shmmax/g > /etc/sysctl.conf.altered
	  rm /etc/sysctl.conf
	  mv /etc/sysctl.conf.altered /etc/sysctl.conf
	fi
	
	# Set the value of shmmax with sysctl so no reboot is necessary    
	echo 'Adjusting the size of shmmax now. No reboot is necessary!'
	/sbin/sysctl -w kernel.shmmax=$new_shmmax 

	# Check to see if shmall needs to be adjusted and do so if it is set to a small value
	if [ $(echo "$current_shmall < $new_shmall" | bc) -eq 1 ]; then
          # If an entry for shmall does not exist in /etc/sysctl.conf add it here
	  if [ -z "$exists_shmall" ]; then
	    echo "kernel.shmall=$new_shmall" >> /etc/sysctl.conf
          else
            # If a current entry for shmall appears in /etc/sysctl.conf and needs to be increased do it here
	    sysctl_conf_shmall=`grep kernel.shmall= /etc/sysctl.conf | sed s/kernel.shmall=//`
            cat /etc/sysctl.conf | sed s/kernel.shmall=$sysctl_conf_shmall/kernel.shmall=$new_shmall/g > /etc/sysctl.conf.altered
	    rm /etc/sysctl.conf
	    mv /etc/sysctl.conf.altered /etc/sysctl.conf
          fi
         
	  # Set the current value of shmall with sysctl now so a reboot is not necessary
	  /sbin/sysctl -w kernel.shmall=$new_shmall
	fi  
      else
        echo ''
	echo 'The file /etc/sysctl.conf does not exist or is not writable by root. Aborting!'
	echo ''
	exit 1
      fi
      
    # If sysctl is not present see if we can set shmmax in /proc/sys/kernel/shmmax and edit rc.local to survive a reboot
    elif [ -e /proc/sys/kernel/shmmax ]; then
      
      # Check that rc.local exists and back it up before continuing
      if [ -e "/etc/rc.d/rc.local" ]; then
        
	# Check to see if /etc/rc.d/rc.local is writable by root
	exists_rc_local=`chmod u+w /etc/rc.d/rc.local | tr "\n" " " | sed s/" "//g`     
        if [ -z "$exists_rc_local" ]; then
          
	  # Backup /etc/rc.d/rc.local
	  DATE=`date +%m%d%Y`
	  if [ -e "/etc/rc.d/rc.local.$DATE" ]; then
	    count=`ls /etc/rc.d/rc.local.$DATE* | wc -l | sed s/" "/""/g`
	    echo "A backup copy of /etc/rc.d/rc.local is being saved as /etc/rc.d/rc.local.$DATE.$count"
	    cp /etc/rc.d/rc.local /etc/rc.d/rc.local.$DATE.$count
	  else
	    echo "A backup copy of /etc/rc.d/rc.local is being saved as /etc/rc.d/rc.local.$DATE"
	    cp /etc/rc.d/rc.local /etc/rc.d/rc.local.$DATE
	  fi

	  # Determine if an entry appears for shmmax in rc.local. If not then add a one.
	  exists_shmmax=`grep "/proc/sys/kernel/shmmax" /etc/rc.d/rc.local | tr "\n" " " | sed s/" "//g`
          
	  # Determine if an entry appears for shmall in rc.local. If not and one is needed add it later.
	  if [ -e "/proc/sys/kernel/shmall" ]; then
	    exists_shmall=`grep "/proc/sys/kernel/shmall" /etc/rc.d/rc.local | tr "\n" " " | sed s/" "//g`
	  else
	    echo 'There does not seem to be a value for shmall set in /proc/sys/kernel/shmall. Aborting!'
	    echo 'Please contact us for further instructions!'
	    exit 1
	  fi
	  
	  # If no entry for shmmax appears in rc.local add one here.
	  if [ -z "$exists_shmmax" ]; then
	    echo '' >> /etc/rc.d/rc.local
	    echo '# Permit large shared memory sections' >> /etc/rc.d/rc.local
	    echo "echo $new_shmmax > /proc/sys/kernel/shmmax" >> /etc/rc.d/rc.local
          else
            
	    # If a current entry for shmmax appears in rc.local edit it here.
	    value_match=`grep $current_shmmax /etc/rc.d/rc.local | tr "\n" " " | sed s/" "//g`
	    if [ -z "$value_match" ]; then
	      echo 'The value to set shmmax in /etc/rc.d/rc.local does not appear to be the same as the'
	      echo 'current shmmax value in /proc/sys/kernel/shmmax. Aborting install!'
	      echo 'Please contact us directly for further instructions'
	      exit 1
	    else
	      cat /etc/rc.d/rc.local | sed s/$current_shmmax/$new_shmmax/g > /etc/rc.d/rc.local.altered
	      rm /etc/rc.d/rc.local
	      mv /etc/rc.d/rc.local.altered /etc/rc.d/rc.local
	    fi
          fi
	  
	  # Set shmmax manually here so we don't have to reboot
	  echo "$new_shmmax" > /proc/sys/kernel/shmmax

	  # Determine if shmall is small and needs to be increased and do so if needed.
          if [ $(echo "$current_shmall < $new_shmall" | bc) -eq 1 ]; then
            if [ -z "$exists_shmall" ]; then
	      
	      # make changes to rc.local
	      echo "echo $new_shmall > /proc/sys/kernel/shmall" >> /etc/rc.d/rc.local
            else
	      
	      # Check if shmall is being set in rc.local already and is different than the current value
	      # This really should be a rare event
	      value_match=`grep $current_shmall /etc/rc.d/rc.local | tr "\n" " " | sed s/" "//g`
              if [ -z "$value_match" ]; then
		echo 'The value to set shmall in /etc/rc.d/rc.local does not appear to be the same as the'
		echo 'current shmall value in /proc/sys/kernel/shmall. Aborting install!'
		echo 'Please contact us directly for further instructions'
		exit 1
	      else
		
		# make changes to rc.local
		cat /etc/rc.d/rc.local | sed s/$current_shmall/$new_shmall/g > /etc/rc.d/rc.local.altered
		rm /etc/rc.d/rc.local
		mv /etc/rc.d/rc.local.altered /etc/rc.d/rc.local
	      fi
            fi
            
	    # Set shmall manually if needed so we don't need to reboot
	    echo "$new_shmall" > /proc/sys/kernel/shmall
          fi
	else
	  echo 'The file /etc/rc.d/rc.local does not exist or is not writable by root. Aborting!'
	fi
      else
        echo 'You seem to be using a version of linux that does not set shmmax in the usual way. Aborting!'
        echo 'Please contact us directly for assistance to increase shmmax'
        exit 1
      fi
    fi
  else
    echo ''
    echo 'You have choosen not to increase the value of shmmax at this time'
  fi
fi

# **********************************************************************************************
# **********************************************************************************************
# Apple OSX specific installation process
if [ $operating_system = "Darwin" ]; then

# Ask if files should be installed
  read -p "STEP 1: Do you want to install RNMRTK files (y/n)?"
  if [ "$REPLY" = "y" ]; then
    
    # Enter installation directory or choose default
    read -p "Enter full path to install directory or hit ENTER for default (/Applications/): "
    if [ -z "$REPLY" ]; then
      install_path="/Applications/"
    elif [ -d "$REPLY" ]; then
      install_path=$REPLY
    else
      echo "The path $REPLY does not exist. Aborting!"
      exit 1
    fi
    
    # Make sure the install directory is write accessable and that the rnmrtk.v3.tar file exists
    if [ ! -e "rnmrtk.v3.tar" ]; then
      echo 'The rnmrtk.v3.tar file is not present. Aborting install!'
      exit 1
    fi

    # Check to see if an existing rnmrtk folder exists as is writable and back it up before proceeding
    exists_install_path=`chmod 755 $install_path 2>&1 | tr "\n" " " | sed s/" "//g`
    if [ -z "$exists_install_path" ]; then
      if [ -d "$install_path/rnmrtk.v3" ]; then
	
	# Ask if we should overwrite existing program
	read -p "rnmrtk.v3 already exists in this folder. Overwrite [Original will be backed-up] (y/n): "
	if [ "$REPLY" = "y" ]; then
	  
	  # Backup the existing program if it exists
	  DATE=`date +%m%d%Y`
	  cd $install_path
	  if [ -e "rnmrtk.v3.$DATE.tar" ]; then
	    count=`ls rnmrtk.v3.$DATE* | wc -l | sed s/" "/""/g`
	    echo "Backing up existing program as $install_path/rnmrtk.v3.$DATE.$count.tar"
	    tar -cf rnmrtk.v3.$DATE.$count.tar rnmrtk.v3
	  else
	    echo "Backing up existing program as $install_path/rnmrtk.v3.$DATE.tar"
	    tar -cf rnmrtk.v3.$DATE.tar rnmrtk.v3
	  fi
	  
	  # Remove the rnmrtk folder after backing it up
	  rm -rf rnmrtk.v3
	  
	  # Copy the rnmrtk program to the install path and unpack, remove the tar file when done.
          cd $current_dir
          cp rnmrtk.v3.tar $install_path
          cd $install_path
          tar -xf rnmrtk.v3.tar
          rm -rf rnmrtk.v3.tar
	else
	  echo 'You have choose not to install files. Proceeding to Step 2'
	  echo ''
	fi
      fi
    else
      echo "Root does not have write access to $install_path. Aborting!"
      exit 1
    fi
  else
    echo ''
    echo 'You have chosen to not install files at this time'
  fi




  # Step 2 edit login files
  
  # Ask if configuration files should be updated
  read -p "STEP 2: Do you want to make changes to global login files for bash/csh/tcsh shells (y/n)?"
  if [ "$REPLY" = "y" ]; then
    
    # Ask for the path to the rnmrtk.v3 folder if no installation was done in step 1
    if [ -z "$install_path" ]; then
      read -p "What is the full path to the rnmrtk.v3 folder (e.g. /usr/local) ? "
      if [ -z "$REPLY" ]; then
	echo 'You did not provide the path. Aborting!'
	exit 1
      else
	install_path=$REPLY
      fi
      
      # Check to see if rnmrtk.v3 was added to the install path and remove it before proceeding if it is present
      exists_rnmrtk=`echo $install_path | grep "/rnmrtk.v3" | tr "\n" " " | sed s/" "//g`
      if [ ! -z "$exists_rnmrtk" ]; then
        install_path=`echo $install_path | sed 's/\/rnmrtk.v3//'`
      fi
    fi
    
    # Find the current stacksize, datasizem and DATE
    stacksize=`ulimit -s`
    datasize=`ulimit -d`
    DATE=`date +%m%d%Y`
    
    # Find the number of cpu's for apple
    cpu_number=`hostinfo | grep 'physically' | awk '{cpus = $1}; END {print cpus}'`
    
   
    # Make changes to /etc/profile
    header="no"
    
    # Check to see if /etc/profile exists and is writable by root
    exists_profile=`chmod 755 /etc/profile 2>&1 | tr "\n" " " | sed s/" "//g`
    if [ -z "$exists_profile" ]; then
      
      # Check to see if the install path is already present in the /etc/profile file. do nothing if it is there already
      exists_path=`cat /etc/profile | sed 's/\/\//\//g' | grep $install_path/rnmrtk.v3 | tr "\n" " " | sed s/" "//g`
      if [ -z "$exists_path" ]; then
        
	# Add a header if any changes are made to /etc/profile
	header="yes"
	echo 'Making changes to /etc/profile'
	echo '' >> /etc/profile
        echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/profile
        
	# Add the path to /etc/profile
	echo "PATH=\"$install_path/rnmrtk.v3:\$PATH\"" >> /etc/profile
        echo 'export PATH' >> /etc/profile
      else
        echo ''
      fi
      
      # Make changes for stack size limit if the stacksize is < 32768
      if [ $(echo "$stacksize < 32768" | bc) -eq 1 ]; then
	
	# Test if stacksize has already been set to 32768 in /etc/profile and do nothing if it has
	exists_stacksize=`grep 'ulimit -s 32768' /etc/profile | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_stacksize" ]; then
	  
	  # Add a header if changes are being made and no header was created earlier
	  if [ ! $header = "yes" ]; then
	    echo 'Making changes to /etc/profile'
	    echo '' >> /etc/profile
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/profile
	    header="yes"
	  fi
	  
	  # Make the change to stacksize
	  echo 'ulimit -s 32768' >> /etc/profile
	fi
      fi
      
      # Make changes for data size to make it unlimited
      if [ ! "$datasize" = "unlimited" ]; then
	
	# Test if datasize has already been set to unlimited in /etc/profile and do nothing if it has
	exists_datasize=`grep 'ulimit -d unlimited' /etc/profile | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_datasize" ]; then
	  
	  # Add a header if changes are being made and no header was created earlier
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/profile'
	    echo '' >> /etc/profile
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/profile
	    header="yes"
	  fi
	  
	  # Make changes to datasize
	  echo 'ulimit -d unlimited' >> /etc/profile
	fi
      fi
      
      # Make changes for mp_set_numthreads
      if [ ! "$cpu_number" = 0 ]; then
	
	# Test if mp_set_numthreads has already been set in /etc/profile and do nothing if it has
	exists_mp_set_numthreads=`grep 'MP_SET_NUMTHREADS' /etc/profile | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_mp_set_numthreads" ]; then
	  
	  # Add a header if changes are being made and no header was created earlier
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/profile'
	    echo '' >> /etc/profile
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/profile
	    header="yes"
	  fi
	  
	  # Make changes to mp_set_numthreads
	  echo "MP_SET_NUMTHREADS=$cpu_number" >> /etc/profile
	  echo 'export MP_SET_NUMTHREADS' >> /etc/profile
	fi
      fi
      
    else
      
      echo 'The file /etc/profile does not exist or is not owned by root'
      echo 'Manual changes to /etc/profile must be made for bash/sh users'
      echo ''
    fi
    
    
    # Make changes to /etc/bashrc
    header="no"
    
    # Test if /etc/bashrc exists and is writable by root
    exists_bashrc=`chmod 755 /etc/bashrc 2>&1 | tr "\n" " " | sed s/" "//g`
    if [ -z "$exists_bashrc" ]; then
      
      # Read /etc/bashrc, remove any "//" which may occur in paths, grep for install path, eliminate redundant rnmrtk paths
      #   into a single line, eliminate any spaces so the exists_path variable can be set
      exists_path=`cat /etc/bashrc | sed 's/\/\//\//g' | grep $install_path/rnmrtk.v3 | tr "\n" " " | sed s/" "//g`
      if [ -z "$exists_path" ]; then
        
	# Print header if changes  are going to be made
	header="yes"
	echo 'Making changes to /etc/bashrc'
	echo '' >> /etc/bashrc
        echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/bashrc
        echo "PATH=\"$install_path/rnmrtk.v3:\$PATH\"" >> /etc/bashrc
        
	# Add the path to /etc/bashrc
	echo 'export PATH' >> /etc/bashrc
      else
        echo ""
      fi
     
      # Make changes for stack size limit if < 32768
      if [ $(echo "$stacksize < 32768" | bc) -eq 1 ]; then
	
	# Determine if stacksize is set in /etc/bashrc already and do nothing if it has	
	exists_stacksize=`grep 'ulimit -s 32768' /etc/bashrc | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_stacksize" ]; then
	  
	  # Add header if changes are being made and no header was added earlier
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/bashrc'
	    echo '' >> /etc/bashrc
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/bashrc
	    header="yes"
	  fi
	  
	  # Make the change to stacksize in /etc/bashrc
	  echo 'ulimit -s 32768' >> /etc/bashrc
	fi
      fi
      
      # Make changes for data size limit if not already set to unlimited
      if [ ! "$datasize" = "unlimited" ]; then
	
	# Determine if datasize is set in /etc/bashrc already and do nothing if it has	
	exists_datasize=`grep 'ulimit -d unlimited' /etc/bashrc | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_datasize" ]; then
	 
	  # Add header if changes are being made and no header was added earlier
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/bashrc'
	    echo '' >> /etc/bashrc
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/bashrc
	    header="yes"
	  fi
	 
	  # Make change to datasize in /etc/bashrc
	  echo 'ulimit -d unlimited' >> /etc/bashrc
	fi
      fi
      
      # Make changes for mp_set_numthreads
      if [ ! "$cpu_number" = 0 ]; then
	
	# Test if mp_set_numthreads has already been set in /etc/bashrc and do nothing if it has
	exists_mp_set_numthreads=`grep 'MP_SET_NUMTHREADS' /etc/bashrc | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_mp_set_numthreads" ]; then
	  
	  # Add a header if changes are being made and no header was created earlier
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/bashrc'
	    echo '' >> /etc/bashrc
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/bashrc
	    header="yes"
	  fi
	  
	  # Make changes to mp_set_numthreads
	  echo "MP_SET_NUMTHREADS=$cpu_number" >> /etc/bashrc
	  echo 'export MP_SET_NUMTHREADS' >> /etc/bashrc
	fi
      fi
      
    else
      echo 'The file /etc/bashrc does not exist or is not owned by root'
      echo 'Manual changes to /etc/bashrc must be made for bash/sh users'
      echo ''
    fi
    
    
    # Make changes to /etc/csh.cshrc
    header="no"
    
    # Test if /etc/csh.cshrc exists and is writable by root
    exists_csh_cshrc=`chmod 755 /etc/csh.cshrc 2>&1 | tr "\n" " " | sed s/" "//g`
    if [ -z "$exists_csh_cshrc" ]; then
      
      # Check if the path already is set in /etc/csh.cshrc and do nothing if it has
      exists_path=`cat /etc/csh.cshrc | sed 's/\/\//\//g' | grep $install_path/rnmrtk.v3 | tr "\n" " " | sed s/" "//g`
      if [ -z "$exists_path" ]; then
        
	# Print header if changes are to be made
	header="yes"
	echo 'Making changes to /etc/csh.cshrc'
	echo '' >> /etc/csh.cshrc
        echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/csh.cshrc
        
	# Set the PATH in /etc/csh.cshrc
	echo "setenv PATH \"$install_path/rnmrtk.v3:\$PATH\"" >> /etc/csh.cshrc
      else
        echo ""
      fi
      
      # Make changes for stack size limit if , 32768
      if [ $(echo "$stacksize < 32768" | bc) -eq 1 ]; then
	
	# Determine if stacksize is already being set in csh.cshrc and do nothing if it has
	exists_stacksize=`grep "limit stacksize 32768" /etc/csh.cshrc | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_stacksize" ]; then
	  
	  # Print the header if not already printed above
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/csh.cshrc'
	    echo '' >> /etc/csh.cshrc
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/csh.cshrc
	    header="yes"
	  fi
	  
	  # Make changes for stacksize in csh.cshrc
	  echo 'limit stacksize 32768' >> /etc/csh.cshrc
	fi
      fi
      
      # Make changes for data size limit if needed
      if [ ! "$datasize" = "unlimited" ]; then
	
	# Test if datasize is already set in csh.cshrc and do nothing if it has
	exists_datasize=`grep "limit datasize unlimited" /etc/csh.cshrc | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_datasize" ]; then
	  
	  # Print header if not already printed above
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/csh.cshrc'
	    echo '' >> /etc/csh.cshrc
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/csh.cshrc
	    header="yes"
	  fi
	  
	  # Make change to datasize in csh.cshrc
	  echo 'limit datasize unlimited' >> /etc/csh.cshrc
	fi
      fi
      
      # Make changes for mp_set_numthreads
      if [ ! "$cpu_number" = 0 ]; then
	
	# Test if mp_set_numthreads has already been set in /etc/csh.cshrc and do nothing if it has
	exists_mp_set_numthreads=`grep 'MP_SET_NUMTHREADS' /etc/csh.cshrc | tr "\n" " " | sed s/" "//g`
        if [ -z "$exists_mp_set_numthreads" ]; then
	  
	  # Add a header if changes are being made and no header was created earlier
	  if [ ! "$header" = "yes" ]; then
	    echo 'Making changes to /etc/csh.cshrc'
	    echo '' >> /etc/csh.cshrc
            echo "# Setup for the Rowland NMR Toolit $DATE" >> /etc/csh.cshrc
	    header="yes"
	  fi
	  
	  # Make changes to mp_set_numthreads
	  echo "setenv MP_SET_NUMTHREADS $cpu_number" >> /etc/csh.cshrc
	fi
      fi
      
    else
      echo '/etc/csh.cshrc does not exist or is not owned by root'
      echo 'Manual changes to /etc/csh.cshrc must be made for csh/tcsh users'
      echo ''
    fi
    
  else
    echo ''
    echo 'You have chosen to not change login files. You must edit the $PATH variable'
    echo 'manually and make changes to the filesize and stacksize if necessary!'
    echo ''
  fi
        
 
 
    
  # Add a slight pause before proceeding to step 3
  sleep 1
  
  # Set shared memory parameters
  
  # Get the current shared memory parameters
  current_shmmax=`/usr/sbin/sysctl kern.sysv.shmmax | awk ' {shmmax = $2}; END {print shmmax} '`
  current_shmmin=`/usr/sbin/sysctl kern.sysv.shmmin | awk ' {shmmin = $2}; END {print shmmin} '`
  current_shmmni=`/usr/sbin/sysctl kern.sysv.shmmni | awk ' {shmmni = $2}; END {print shmmni} '`
  current_shmseg=`/usr/sbin/sysctl kern.sysv.shmseg | awk ' {shmseg = $2}; END {print shmseg} '`
  current_shmall=`/usr/sbin/sysctl kern.sysv.shmall | awk ' {shmall = $2}; END {print shmall} '`
  
  # Determine the processor type and OS version
  os_version=`uname -v | awk ' {version = $4}; END {printf "%3.1f", version}'`
  echo ''
  echo "The OS kernel version is $os_version"
  echo ''
  
  # Report how large the shared memory parameters are
  echo ''
  echo '************************** Information on current shared memory parameters **************************'
  echo "The current setting for the boot parameter shmmax is $current_shmmax bytes ($(($current_shmmax/(1024*1024)))) MB"
  echo "With this setting the largest NMR data set that may be processed would be slightly less than $(($current_shmmax/(1024*1024))) MB"
  echo "     That is the size of a typical 2D experiment with $(($current_shmmax/(2048*4*2))) final points in the indirect dimension or"
  x=`echo "sqrt(($current_shmmax/(1024*4*4)))" | bc`
  echo "     a typcial 3D experiment with an average of $x final points in the indirect dimensions."
  echo ' NOTE: Increasing the value of shmmax does not affect the amount of usable RAM on your system'
  echo ''
  
  # Ask if the shared memory parameters should be increased and wait for reply
  read -p "STEP 3: Do you want to increase the size of shmmax at this time (y/n)?"
  if [ "$REPLY" = "y" ]; then
    read -p "Choose the shared memory size you would like: (1)256 MB, (2)512 MB, (3)768 MB, (4)1 G, (5)2 G, or (6)4G: "
    if [ "$REPLY" = "1" ]; then
      new_shmmax=272629760
      new_shmall=66560
    elif [ "$REPLY" = "2" ]; then
      new_shmmax=541065216
      new_shmall=132096
    elif [ "$REPLY" = "3" ]; then
      new_shmmax=809500672
      new_shmall=197632
    elif [ "$REPLY" = "4" ]; then
      new_shmmax=1077936128
      new_shmall=263168
    elif [ "$REPLY" = "5" ]; then
      new_shmmax=2151677952
      new_shmall=525312
    elif [ "$REPLY" = "6" ]; then
      new_shmmax=4293918720
      new_shmall=1048320
    else
      echo 'An invalid entry was entered. Aborting install!'
      exit 1
    fi 
    
    # Checking to see if new shmmax value choose is larger than the existing shmmax value.
    if [ $(echo "$current_shmmax == $new_shmmax" | bc) -eq 1 ]; then
      echo 'The current shmmax value is the same as the requested value. No change is necessary'
      echo ''
      echo 'Installation complete!'
      echo ''
      exit 1
    fi
    
    # Ask if the user should continue if the user is trying to make shmmax smaller than existing setting
    if [ $(echo "$current_shmmax > $new_shmmax" | bc) -eq 1 ]; then
      read -p "The current shmmax value is larger than the requested size. Do you want to continue and make shmmax smaller (y/n)?"
      if [ "$REPLY" = "y" ]; then
        echo 'Adjusting shmmax to a smaller value'
      else
	echo ''
	echo 'Installation compelte!'
	echo ''
	exit 1
      fi
    fi
      
    # If kernel version is less than 7.0 10.2 or before abort as it is not supported here
    if [ $(echo "$os_version < 7.0" | bc) -eq 1 ]; then
      echo 'This script only works on OS X 10.3 and later. Please contact us if you need to adjust shmmax for an earlier version of OS X'
      echo 'Installation is proceeding, but shared memory parameters will need to be adjusted manually!'
    fi
    
    # If kernel version is 10.3 - 10.3.8 or less request that an upgrade to 10.3.9. If not adjust shmmax in /etc/rc
    if [ $(echo "$os_version < 7.9" | bc) -eq 1 ]; then
      echo 'Adjustment of shared memory parameters is more robust in version 10.3.9 than in ealier Panther versions'
      echo 'It is better to update your version of OS X to the latest release by going to the Software Update from'
      echo 'the Apple Icon before proceeding'
      
      # Ask if an upgrade should proceed updating the parameters and wait for reply
      read -p "Do you want to proceed with updating shared memory paramters now (y) or run the Software Update (n) first (y/n)?"
      if [ "$REPLY" = "y" ]; then
	echo 'These updates will be reset after you run Software Update and will need to be adjusted again after the Update'
	echo 'It is better to run Software Update first before running this install script to avoid this hassle later'
	
	# For kernel version 7.8 and less shmmax should be set in /etc/rc so this should work reliably.
	# Make a backup copy of /etc/rc before proceeding
	DATE=`date +%m%d%Y`
	echo "A backup copy of /etc/rc is being saved as /etc/rc.$DATE"
	cp /etc/rc /etc/rc.$DATE
        
	# Make changes to /etc/rc
	chmod 644 /etc/rc
	cat /etc/rc | sed s/kern.sysv.shmmax=$current_shmmax/kern.sysv.shmmax=$new_shmmax/g > /etc/rc.altered
	rm /etc/rc
	mv /etc/rc.altered /etc/rc
        cat /etc/rc | sed s/kern.sysv.shmall=$current_shmall/kern.sysv.shmall=$new_shmall/g > /etc/rc.altered
	rm /etc/rc
	mv /etc/rc.altered /etc/rc
      else 
        echo 'Aborting install! Please run the install script again after running Software Update'
	exit 1
      fi
    fi
    
    # For 10.3.9 and later update shmmax in /etc/sysctl.conf. Note all 5 shared memory paramters must exist
    #    in /etc/sysctl.conf or none will be read
    if [ $(echo "$os_version >= 7.9" | bc) -eq 1 ]; then
      echo "Updating shared memory parameters now!"
      
      # Make backup copy of sysctl.conf before proceeding
      if [ -e "/etc/sysctl.conf" ]; then
	DATE=`date +%m%d%Y`
	if [ -e "/etc/sysctl.conf.$DATE" ]; then
	  count=`ls /etc/sysctl.conf.$DATE* | wc -l | sed s/" "/""/g`
	  echo "A backup copy of /etc/sysctl.conf is being saved as /etc/sysctl.conf.$DATE.$count"
	  cp /etc/sysctl.conf /etc/sysctl.conf.$DATE.$count
	else
	  echo "A backup copy of /etc/sysctl.conf is being saved as /etc/sysctl.conf.$DATE"
	  cp /etc/sysctl.conf /etc/sysctl.conf.$DATE
	fi
      else
        
	# Create sysctl.conf if it does not exist
	touch /etc/sysctl.conf
      fi
      
      # Check to see if sysctl.conf exists and is writable by root
      exists_sysctl_conf=`chmod 644 /etc/sysctl.conf | tr "\n" " " | sed s/" "//g`
      if [ -z "$exists_sysctl_conf" ]; then
      
	# Check to see if entries for shared memory parameters exists before continuing
	exists_shmmax=`grep kern.sysv.shmmax= /etc/sysctl.conf | tr "\n" " " | sed s/" "//g`
	exists_shmmin=`grep kern.sysv.shmmin= /etc/sysctl.conf | tr "\n" " " | sed s/" "//g`
	exists_shmmni=`grep kern.sysv.shmmni= /etc/sysctl.conf | tr "\n" " " | sed s/" "//g`
	exists_shmseg=`grep kern.sysv.shmseg= /etc/sysctl.conf | tr "\n" " " | sed s/" "//g`
	exists_shmall=`grep kern.sysv.shmall= /etc/sysctl.conf | tr "\n" " " | sed s/" "//g`
	
	# If not shmmax entry exists in sysctl.conf add one here
	if [ -z "$exists_shmmax" ]; then
	  echo "kern.sysv.shmmax=$new_shmmax" >> /etc/sysctl.conf
	else
         
	  # If shmmax appears in sysctl.conf update the value with the new larger value
	  sysctl_conf_shmmax=`grep kern.sysv.shmmax= /etc/sysctl.conf | sed s/kern.sysv.shmmax=//`
          cat /etc/sysctl.conf | sed s/kern.sysv.shmmax=$sysctl_conf_shmmax/kern.sysv.shmmax=$new_shmmax/g > /etc/sysctl.conf.altered
	  rm /etc/sysctl.conf
	  mv /etc/sysctl.conf.altered /etc/sysctl.conf
	fi
	
	# Add entries for shmmin, shmmni, and shmseg if they do not already exist. If they exist leave them alone.
	if [ -z "$exists_shmmin" ]; then
	  echo "kern.sysv.shmmin=$current_shmmin" >> /etc/sysctl.conf
	fi
	if [ -z "$exists_shmmni" ]; then
	  echo "kern.sysv.shmmni=$current_shmmni" >> /etc/sysctl.conf
	fi
	if [ -z "$exists_shmseg" ]; then
	  echo "kern.sysv.shmseg=$current_shmseg" >> /etc/sysctl.conf
	fi
	
	# Add an entry for shmall or update the shmall parameter if it already exists.
	if [ -z "$exists_shmall" ]; then
	  echo "kern.sysv.shmall=$new_shmall" >> /etc/sysctl.conf
	else
          sysctl_conf_shmall=`grep kern.sysv.shmall= /etc/sysctl.conf | sed s/kern.sysv.shmall=//`
	  cat /etc/sysctl.conf | sed s/kern.sysv.shmall=$sysctl_conf_shmall/kern.sysv.shmall=$new_shmall/g > /etc/sysctl.conf.altered
	  rm /etc/sysctl.conf
	  mv /etc/sysctl.conf.altered /etc/sysctl.conf
	fi
      else
        echo 'The file /etc/sysctl.conf does not exist or is not writable by root. Aborting!'
	exit 1
      fi
    fi
  else
    echo ''
    echo 'You have choosen not to change the shared memory parameters at this time'
  fi
fi
echo ''
echo 'Installation script is complete'
echo ''

exit 1
